@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2020 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -365,7 +365,8 @@ int FileCopy::Do()
 	 return MOVED;
       }
    pre_CONFIRM_WAIT:
-      put->SetSuggestedFileName(get->GetSuggestedFileName());
+      if(put->IsAutoRename())
+	 put->SetSuggestedFileName(get->GetSuggestedFileName());
       put->SetDate(get->GetDate());
       if(get->GetSize()!=NO_SIZE && get->GetSize()!=NO_SIZE_YET)
 	 put->SetEntitySize(get->GetSize());
@@ -808,6 +809,11 @@ const char *FileCopyPeer::UseTempFile(const char *file)
    return temp;
 }
 
+bool FileCopyPeer::ShouldRename() const
+{
+   return (auto_rename || temp_file) && suggested_filename;
+}
+
 FileCopyPeer::FileCopyPeer(dir_t m) : IOBuffer(m)
 {
    want_size=false;
@@ -909,7 +915,7 @@ int FileCopyPeerFA::Do()
       }
       else if(verify->Done())
       {
-	 if(suggested_filename && auto_rename)
+	 if(ShouldRename())
 	 {
 	    const char *new_name=dir_file(dirname(file),suggested_filename);
 	    bool clobber=temp_file;
@@ -1407,6 +1413,14 @@ FileCopyPeer *FileCopyPeerFA::Clone()
    return c;
 }
 
+const char *FileCopyPeerFA::UseTempFile(const char *file)
+{
+   const char *temp=FileCopyPeer::UseTempFile(file);
+   if(temp!=file && orig_url)
+      dirname_modify(orig_url).append('/').append_url_encoded(basename_ptr(temp),URL_PATH_UNSAFE);
+   return temp;
+}
+
 // FileCopyPeerFDStream
 #undef super
 #define super FileCopyPeer
@@ -1492,7 +1506,7 @@ void FileCopyPeerFDStream::Seek_LL()
 
 int FileCopyPeerFDStream::getfd()
 {
-   if(!stream)
+   if(do_mkdir || !stream)
       return -1;
    if(stream->fd!=-1)
       return stream->fd;
@@ -1537,7 +1551,7 @@ int FileCopyPeerFDStream::Do()
       }
       else if(verify->Done())
       {
-	 if(suggested_filename && stream && stream->full_name && auto_rename)
+	 if(ShouldRename() && stream && stream->full_name)
 	 {
 	    const char *new_name=dir_file(dirname(stream->full_name),suggested_filename);
 	    struct stat st;
