@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,8 +32,7 @@ char *xstrdup(const char *s,int spare=0);
 char *xstrset(char *&mem,const char *s);
 char *xstrset(char *&mem,const char *s,size_t n);
 #define alloca_strdup(s) alloca_strdup2((s),0)
-#define alloca_strdup2(s,n) ((s)?strcpy((char*)alloca(strlen((s))+1+(n)),(s)) \
-                                :((n)==0?0:(char*)alloca((n))))
+#define alloca_strdup2(s,n) (strcpy((char*)alloca(strlen((s))+1+(n)),(s)))
 #define alloca_append(s1,s2) strcat(alloca_strdup2((s1),strlen((s2))),(s2));
 
 void xfree(void *p);
